package com.ruoyi.business.mapper;

import java.util.List;
import com.baomidou.mybatisplus.core.mapper.BaseMapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.Constants;
import org.apache.ibatis.annotations.*;
import com.ruoyi.business.domain.FileExchangePackage;

/**
 * 对外文件打包Mapper接口
 * 
 * @author ruoyi
 * @date 2024-01-29
 */
public interface FileExchangePackageMapper extends BaseMapper<FileExchangePackage> {

    /**
     * 查询对外文件打包
     * 
     * @param id 对外文件打包主键
     * @return 对外文件打包
     */
    FileExchangePackage selectFileExchangePackageById(Long id);

    /**
     * 查询对外文件打包列表
     * 
     * @param fileExchangePackage 对外文件打包
     * @return 对外文件打包集合
     */
    List<FileExchangePackage> selectFileExchangePackageList(FileExchangePackage fileExchangePackage);

    /**
     * 新增对外文件打包
     * 
     * @param fileExchangePackage 对外文件打包
     * @return 结果
     */
    int insertFileExchangePackage(FileExchangePackage fileExchangePackage);

    /**
     * 修改对外文件打包
     * 
     * @param fileExchangePackage 对外文件打包
     * @return 结果
     */
    int updateFileExchangePackage(FileExchangePackage fileExchangePackage);

    /**
     * 删除对外文件打包
     * 
     * @param id 对外文件打包主键
     * @return 结果
     */
    int deleteFileExchangePackageById(Long id);

    /**
     * 批量删除对外文件打包
     * 
     * @param ids 需要删除的数据主键集合
     * @return 结果
     */
    int deleteFileExchangePackageByIds(Long[] ids);

    /**
     * 批量删除对外文件打包（逻辑）
     * @param updateWrapper
     */
     @Update("update b_file_exchange_package set ${ew.sqlSet} ${ew.customSqlSegment}")
     int updateByIds(@Param(Constants.WRAPPER) LambdaUpdateWrapper<FileExchangePackage> updateWrapper);
}
